/*  
 Date: 10/30/2011
 Author: Honglei Liu <liuhonglei@gmail.com>

 Description: 
 */

#include <iostream>
#include <sstream>
#include <cassert>
#include <sys/time.h>
#include <stdlib.h>

#include "Aligner.h"
#include "BWT.h"
#include "Parameters.h"
#include "pattern.h"
#include "Text.h"
#include "Tree.h"
#include "input.h"
#include "output.h"

using namespace std;

void PrintShortDesc() {
	
	printf("\n");
	printf("ALAE v1.0, Copyright (C) 2011, Northeastern Universtiy.\n");
	printf("ALAE comes with ABSOLUTELY NO WARRENTY.\n");
	printf("ALAE is free software, and you are welcome to\n");
	printf("redistribute it under certain conditions.\n");
	printf("For details type alae.\n");
	printf("\n");

}

void PrintHelp() {

	printf("ALAE v1.0, Copyright (C) 2011, Northeastern Universtiy.\n");
	printf("\n");

	printf("This program is free software; you can redistribute it and/or\n");
	printf("modify it under the terms of the GNU General Public License\n");
	printf("as published by the Free Software Foundation; either version 2\n");
	printf("of the License, or (at your option) any later version.\n");
	printf("\n");

	printf("This program is distributed in the hope that it will be useful,\n");
	printf("but WITHOUT ANY WARRANTY; without even the implied warranty of\n");
	printf("MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n");
	printf("GNU General Public License for more details.\n");
	printf("\n");

	printf("You should have received a copy of the GNU General Public License\n");
	printf("along with this program; if not, write to the Free Software\n");
	printf("Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.\n");
	printf("\n");

	printf("Syntax: alae <index file> <query file> <output file>\n");
	printf("               -H <cutoff score>\n");
	printf("               -R <reward for a match>\n");
	printf("               -Q <penalty for a mismatch>\n");
	printf("               -G <cost to open a gap>\n");
	printf("               -E <cost to extend a gap>\n");
}


void aligner(string index_name, string query_name, string output_name)
{
double totalTime = 0;
struct timeval tv1, tv2;
struct timezone tz;

    string p;
    readString(p,query_name);
    Aligner aligner(index_name,p);

    cout<<endl;
    cout<<"Index File      : "<<index_name<<endl;
    cout<<"Query File      : "<<query_name<<endl;
    cout<<endl;

gettimeofday(&tv1, &tz);

    aligner.Alignment();

gettimeofday(&tv2, &tz);

    aligner.PrintParam();

totalTime = (tv2.tv_sec - tv1.tv_sec)  + (double)(tv2.tv_usec - tv1.tv_usec)/1000000;
cout<<"Alignment time:"<<totalTime<<" Seconds"<<endl<<endl;

    aligner.Output(output_name);

    cout<<endl;
}

void ParseInput(string para, int value)
{
	if(para=="-H")
	{
		Parameters::thresholdH=value;
	}
	else if(para=="-R")
	{
		Parameters::matchScore=value;
	}
	else if(para=="-Q")
	{
		Parameters::mismatchScore=value;
	}
	else if(para=="-G")
	{
		Parameters::gapOpenScore=value;
	}
	else if(para=="-E")
	{
		Parameters::gapExtendScore=value;
	}
	
}

int main(int argc, char* argv[])
{
	PrintShortDesc();

	if(argc > 3)
	{
		int i=4;
		while(i+1<argc)
		{
			ParseInput(string(argv[i]),atoi(argv[i+1]));
			i+=2;
		}

		aligner(string(argv[1]),string(argv[2]),string(argv[3]));
	
	}
        else
        {
            PrintHelp();
        }
	
}
