#include <string>
#include <iostream>
#include <fstream>
#include <stdlib.h>
#include <assert.h>
#include <vector>
#include <algorithm>
#include <math.h>
#include <tr1/unordered_set>
#include "dominate.h"
#include "../BWT.h"
#include "../Parameters.h"



using namespace std;
using namespace tr1;

unordered_set<pair<unsigned int,unsigned int>,pair_hashh,compare_pair> m_dominateTable;
void write_to_disk(const string & filename ,unordered_set<pair<unsigned int,unsigned int>,pair_hashh,compare_pair>& table);
void read_from_disk(const string & filename ,unordered_set<pair<unsigned int,unsigned int>,pair_hashh,compare_pair>& table);

void PrintShortDesc() {
	
	printf("\n");
	printf("dominate build, Copyright (C) 2011, Northeastern Universtiy.\n");
	printf("\n");

}

void PrintHelp() {

	printf("This program is free software; you can redistribute it and/or\n");
	printf("modify it under the terms of the GNU General Public License\n");
	printf("as published by the Free Software Foundation; either version 2\n");
	printf("of the License, or (at your option) any later version.\n");
	printf("\n");

	printf("This program is distributed in the hope that it will be useful,\n");
	printf("but WITHOUT ANY WARRANTY; without even the implied warranty of\n");
	printf("MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n");
	printf("GNU General Public License for more details.\n");
	printf("\n");

	printf("You should have received a copy of the GNU General Public License\n");
	printf("along with this program; if not, write to the Free Software\n");
	printf("Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.\n");
	printf("\n");

	printf("Syntax: dominate_build <bwt index file>\n");
}

static inline int char2int(char c)
{
	return Parameters::GetCharNum(c);
}
static inline char int2char(int c)
{
	switch(c)
	{
	case 0:
		return 'A';
	case 1:
		return 'R';
	case 2:
		return 'N';
	case 3:
		return 'D';
	case 4:
		return 'C';
	case 5:
		return 'Q';
	case 6:
		return 'E';
	case 7:
		return 'G';
	case 8:
		return 'H';
	case 9:
		return 'I';
	case 10:
		return 'L';
	case 11:
		return 'K';
	case 12:
		return 'M';
	case 13:
		return 'F';
	case 14:
		return 'P';
	case 15:
		return 'S';
	case 16:
		return 'T';
	case 17:
		return 'W';
	case 18:
		return 'Y';
	case 19:
		return 'V';
	default:
		cerr<<"int \""<<c<<"\" err."<<endl;
		return -1;
	}
}
static int gram2index(const string & gram)
{
	assert(gram.size()==Parameters::qLen);
	int ret=0;
	for (unsigned int i = 0; i < Parameters::qLen; i += 1)
	{
		ret *= ALPHABETSIZE;
		ret +=char2int(gram[i]); 
	}
	return ret;
}
static string index2gram(int index)
{
	char buf[Parameters::qLen+1];
	for (unsigned int i = 0; i < Parameters::qLen; i += 1)
	{
		buf[Parameters::qLen-i-1] = int2char(index % ALPHABETSIZE);
		index /= ALPHABETSIZE;
	}
	buf[Parameters::qLen]='\0';
	return string(buf);
}
static bool dominateCalDo_(const vector<unsigned int>& posA,const vector<unsigned int>& posB)
{
	unsigned int posALen = posA.size();
	unsigned int posBLen = posB.size();
	assert(posALen>posBLen);
	unsigned int i=0,j=0;
	//vector<unsigned int>::const_iterator j=posA.begin();
	//cout<<"================================\n";
	for ( ; i < posBLen; i += 1)
	{
		int tmp = posB[i]-1;
		/*j = upper_bound(j,posA.end(),tmp);
		if (j==posA.end() || (*j)!=tmp )
		{
			return false;
		}*/
		if (j>=posALen)
		{
			return false;
		}
		for (; j < posALen; j += 1)
		{
			if (tmp>posA[j])
			{
				continue;
			}
			else if (tmp==posA[j])
			{
				break;
			}
			else return false;
		}
	}
//	cout<<"yes"<<endl;
	return true;
}
static int dominateCalDo(const vector<unsigned int>& posA,const vector<unsigned int>& posB)
{
	unsigned int posALen = posA.size();
	unsigned int posBLen = posB.size();
	if (posALen!=0 && posBLen!=0)
	{
		if (posALen > posBLen)
		{
			if (dominateCalDo_(posA,posB))
			{
				return 1;
			}
			return -1;
		}
		else if (posBLen > posALen)
		{
			if (dominateCalDo_(posB,posA))
			{
				return 2;
			}
			return -1;
		}
		else // posBLen == posALen
		{
			int val = posA[0]-posB[0];
			if (val==1||val==-1)
			{
				for (unsigned int i = 0; i < posALen; i += 1)
				{
					if (val != posA[i]-posB[i])
					{
						return -1;
					}
				}
				if (val==1)
				{
					return 2;
				}
				return 1;
			}
			else return -1;
		}
	}
	return -1;
	if (posALen == 0 && posBLen == 0)
	{
		return 1;
	}
	else if (posALen == 0)
	{
		return 2;
	}
	else if (posBLen == 0)
	{
		return 1;
	}
}
void dominatePreProcess(const string & name)
{
	cout<<"dominatePreProcess begin..."<<endl;
	assert(ALPHABETSIZE<=20);
	BWT bwt(name);
	vector< vector<unsigned int> *> buffer;
	unsigned int ALL_GRAM_LEN = pow(ALPHABETSIZE,Parameters::qLen);
	cout<<"ALL_GRAM_LEN = "<<ALL_GRAM_LEN<<endl;
	for (unsigned int i = 0; i < ALL_GRAM_LEN; i += 1)
	{
		vector<unsigned int>* pos = bwt.GetPosList(index2gram(i));
		if (pos)
		{
			sort(pos->begin(),pos->end());
		}
		buffer.push_back(pos);
	}
	cout<<"sort done..."<<endl;
	for (unsigned int i = 0; i < ALL_GRAM_LEN; i += 1)
	{
		//cout<<",,,,,,,,,,,,\n";
		if (i%(ALL_GRAM_LEN/10)==0)
		{
			cout<<i<<"\t------\t";
			cout<<m_dominateTable.size()<<endl;
		}
		for (unsigned int j = 0; j <= i; j+= 1)
		{
			//cout<<"i="<<i<<" j="<<j<<endl;
			int ret = dominateCalDo( *(buffer[i]) , *(buffer[j]) );
			if (ret == -1)
			{
				//mTable[i][j] = mTable[j][i] = 0;
			}
			if (ret== 1)
			{
				//cout<<"A dominate find..."<<endl;
				//cout<<index2gram(i)<<"-->"<<index2gram(j)<<endl;
				//mTable[i][j] = 1;
				m_dominateTable.insert(pair<unsigned int,unsigned int>(i,j));
				//mTable[j][i] = 0;
			}
			if (ret== 2)
			{
				//cout<<"A dominate find..."<<endl;
				//cout<<index2gram(j)<<"-->"<<index2gram(i)<<endl;
				//mTable[j][i] = 1;
				m_dominateTable.insert(pair<unsigned int,unsigned int>(j,i));
				//mTable[i][j] = 0;
			}
		}
	}
	string name2=name+".dominate";
	write_to_disk(name2,m_dominateTable);
	cout<<"dominatePreProcess end..."<<endl;
} 
bool dominateCheck(const string & gramA,const string & gramB)
{
	// A-->B
	if
	(
	m_dominateTable.end() 
	==
	m_dominateTable.find(pair<unsigned int,unsigned int>(gram2index(gramA),gram2index(gramB)))
	)
	{return false;}
	return true;
}
void read_from_disk(const string & filename ,unordered_set<pair<unsigned int,unsigned int>,pair_hashh,compare_pair>& table)
{
	ifstream fin(filename.c_str());
	if (!fin)
	{
		cerr<<"open file\""<<filename<<"\" fail...\n";
	}
	while(fin)
	{
		unsigned int a,b;
		fin>>a;
		fin>>b;
		table.insert(pair<unsigned int,unsigned int>(a,b));
	}	
}
void write_to_disk(const string & filename ,unordered_set<pair<unsigned int,unsigned int>,pair_hashh,compare_pair>& table)
{
	ofstream fout(filename.c_str());
	if (!fout)
	{
		cerr<<"open file\""<<filename<<"\" fail...\n";
	}
	unordered_set<pair<unsigned int,unsigned int>,pair_hashh,compare_pair>::iterator i,j=table.end();
	for (i = table.begin(); i !=j; i++)
	{
		fout<<i->first<<" "<<i->second<<" ";
	}	
}
int main(int argc, char* argv[])
{
	PrintShortDesc();

	if(argc!=2)
           PrintHelp();
        else 
            dominatePreProcess(argv[1]);



}






